package mima.domain;

import java.util.ArrayList;
import java.util.Collection;


/**
 * INV: Invariant from super class
 * <br/>
 * INV: <code>getSubNodes() != null</code>
 * 
 * @author Virginio Carfagno
 */
public class SubNode extends Node {

	private static final Frame DEFAULT_FRAME = Frame.UNDERLINE;
	private static final String DEFAULT_TITLE = "";
	
	
	private Collection _subNodes;
	private Node _superNode;
	
	
	public SubNode(Node superNode) {
		super(DEFAULT_TITLE, DEFAULT_FRAME);
		if (superNode == null)
			throw new IllegalArgumentException("superNode can't be null");
		
		_subNodes = new ArrayList();
		setSuperNode(superNode);
		
		assert _superNode == superNode;
		invariant();
	}


	protected void doAdd(SubNode subNode) {
		checkNotNull(subNode);
		invariant();
		assert !_subNodes.contains(subNode);
		
		_subNodes.add(subNode);
		
		assert _subNodes.contains(subNode);
		invariant();
	}
	
	protected void doRemove(SubNode subNode) {
		checkNotNull(subNode);
		invariant();
		
		_subNodes.remove(subNode);
		
		assert !_subNodes.contains(subNode);
		invariant();
	}

	/**
	 * This method doesn't return <code>_subNodes</code> directly to ensure
	 * the encapsulation and consistency of this object. A manipulation to the returned 
 	 * <code>Collection</code> doesn't have any effects on this object. To add 
 	 * or remove a <code>SubNode</code> to this node you should use the method 
 	 * <code>setSuperNode(Node)</code> or <code>removeFromSuperNode()</code>
 	 * from <code>SubNode</code>.
	 */
	public Collection getSubNodes() {
		return new ArrayList(_subNodes); 
	}

	public Node getSuperNode() {
		return _superNode;
	}
	
	protected void invariant() {
		super.invariant();
		assert getSubNodes() != null;
	}

	public boolean isRight() {
		return _superNode.isRight(this);
	}
	
	protected boolean isRight(SubNode subNode) {
		return _superNode.isRight(this);
	}

	/**
	 * POST: getSuperNode() == null;
	 */
	public void removeFromSuperNode() {
		invariant();
		
		setSuperNode(null);
	
		assert getSuperNode() == null;
		invariant();
	}
	
	/**
	 * POST: <code>getSuperNode() == superNode</code>
	 */
	public void setSuperNode(Node superNode) {
		invariant();
		
		if (_superNode != null)
			_superNode.remove(this);
		_superNode = superNode;
		if (_superNode != null)
			_superNode.add(this);
		
		assert getSuperNode() == superNode;
		invariant();
	}

}
