package mima.presentation.gwt.mindmap;

import mima.domain.CentralNode;
import mima.presentation.gwt.node.CentralNodeView;
import mima.presentation.gwt.node.NodeView;

import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.HasHorizontalAlignment.HorizontalAlignmentConstant;


/**
 * @author Virginio Carfagno
 */
public class MindMap extends Tree {

	private AbsolutePanel _leftEdges;
	private VerticalPanel _leftSubTrees;
	
	private AbsolutePanel _rightEdges;
	private VerticalPanel _rightSubTrees;
	
	
	public MindMap() {
		super(new CentralNode());
		
		_rightSubTrees = createRightSubTreesContainer();
		_leftSubTrees = createLeftSubTreesContainer();
		
		_rightEdges = createEdgesContainer();
		_leftEdges = createEdgesContainer();
		
		composeMindMap();
	}
	
	
	private void composeMindMap() {
		getTree().add(_leftSubTrees);
		getTree().add(_leftEdges);
		getTree().add(getNodeView());
		getTree().add(_rightEdges);
		getTree().add(_rightSubTrees);
	}
	
	private VerticalPanel createLeftSubTreesContainer() {
		return createSubTreesContainer(HasHorizontalAlignment.ALIGN_RIGHT);
	}
	
	private VerticalPanel createRightSubTreesContainer() {
		return createSubTreesContainer(HasHorizontalAlignment.ALIGN_LEFT);
	}
	
	private VerticalPanel createSubTreesContainer(HorizontalAlignmentConstant alignment) {
		VerticalPanel subTrees = new VerticalPanel();
		subTrees.setHorizontalAlignment(alignment);
		return subTrees;
	}
	
	protected void expand(boolean value) {
		_rightSubTrees.setVisible(value);
		_rightEdges.setVisible(value);
		_leftSubTrees.setVisible(value);
		_leftEdges.setVisible(value);
	}
	
	private CentralNode getCentralNode() {
		return (CentralNode)getNode();
	}

	/**
	 * This method was introduced for testing with Pattern Window Driver.
	 */
	public String getCentralNodeTitle() {
		return getNodeView().getNodeTitle();
	}

	protected void redrawAllEdges() {
		SubTree.redrawAllEdges(_rightEdges, _rightSubTrees);
		SubTree.redrawAllEdges(_leftEdges, _leftSubTrees);
	}
	
	protected void updateTreeStructure() {
		SubTree.updateTreeStructure(this, getCentralNode().getRightSubNodes(), _rightSubTrees);
		SubTree.updateTreeStructure(this, getCentralNode().getLeftSubNodes(), _leftSubTrees);
	}

	protected NodeView createRoot() {
		return new CentralNodeView(getCentralNode());
	}

}
