package mima.presentation.gwt.mindmap;

import mima.domain.Node;
import mima.domain.NodeObserver;
import mima.presentation.gwt.node.NodeView;

import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;


/**
 * @author Virginio Carfagno
 */
abstract class Tree extends Composite implements NodeObserver {

	protected static final int EDGES_WIDTH = 40;
	
	private Node _node;
	
	private NodeView _root;
	
	private HorizontalPanel _tree;
	
	
	protected Tree(Node node) {
		_node = node;
		_tree = new HorizontalPanel();
		_tree.setVerticalAlignment(HasVerticalAlignment.ALIGN_MIDDLE);
		initWidget(_tree);
		
		_root = createRoot();
		
		// this wird nach _nodeView als Observer von Node registriert, damit
		// this immer nach NodeView aktualisiert wird.
		_node.registerObserver(this);
	}
	
	
	protected AbsolutePanel createEdgesContainer() {
		AbsolutePanel edges = new AbsolutePanel();
		edges.setWidth(EDGES_WIDTH + "px");
		edges.setHeight("100%");
		return edges;
	}
	
	protected abstract NodeView createRoot();
	
	protected abstract void expand(boolean value);
	
	protected int getAbsoluteEdgeConnectionPointY() {
		return _root.getAbsoluteEdgeConnectionPointY();
	}
	
	protected Node getNode() {
		return _node;
	}
	
	protected NodeView getNodeView() {
		return _root;
	}

	protected HorizontalPanel getTree() {
		return _tree;
	}

	public void nodeChanged() {
		updateView();
	}
	
	/**
	 * This method is overriden to syhchronize the view with the model when it
	 * first appears on the screen.
	 */
	protected void onLoad() {
		super.onLoad();
		updateView();
	}
	
	protected abstract void redrawAllEdges();
	
	protected abstract void updateTreeStructure();
	
	private void updateView() {
		updateTreeStructure();
		expand(_node.isExpanded());
		redrawAllEdges();
	}
	
}
