package mima.presentation.gwt.nodeframes;

import mima.presentation.gwt.nodecomponent.NodeComponent;
import mima.presentation.gwt.nodecomponent.NodeFrame;
import mima.presentation.gwt.util.WidgetUtilities;

import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.Widget;
import com.gwt.components.client.Canvas;


/**
 * @author Virginio Carfagno
 */
// TODO Eigentlich mssten noch die Methoden setWidth(), setTitle() an die
// Komponente delegiert werden. Da fr einen Knoten diese Eigenschaften 
// momentan sowieso nicht gesetzt werden, ist dies erstmal unwichtig.
public class CircleFrame extends NodeFrame {

	private static final int PADDING = 20; 
	
	private Widget _circle;
	private AbsolutePanel _panel;
	
	
	public CircleFrame(NodeComponent nodeComponent) {
		super(nodeComponent);
		_panel = new AbsolutePanel();
		_panel.add(nodeComponent);
		initWidget(_panel);
	}
	

	private int calculateCircleDiameter() {
		return (getNodeComponent().getOffsetWidth() > getNodeComponent()
				.getOffsetHeight() ? getNodeComponent().getOffsetWidth()
				: getNodeComponent().getOffsetHeight())
				+ PADDING;
	}

	private int calculateNodeComponentCenterPosition() {
		return calculateRadius() - (getNodeComponent().getOffsetWidth() / 2);
	}

	private int calculateNodeComponentMiddlePosition() {
		return calculateRadius() - (getNodeComponent().getOffsetHeight() / 2);
	}

	private int calculateRadius() {
		return calculateCircleDiameter() / 2;
	}
	
	private Widget createCircle() {
		int radius = calculateRadius();
		int diameter = radius * 2;
		Canvas canvas = new Canvas(diameter, diameter);
		
		// Der echte Radius des Kreises darf nicht genau so gro wie die
		// Zeichenflche sein, weil der Kreis sonst nicht vollstndig
		// dargestellt wird. (daher die -1)
		canvas.arc(radius, radius, radius - 1, 0,
				(float) (2 * Math.PI), false);
		canvas.setStrokeColor("blue");
		canvas.stroke();

		// Es mssen zwei Kreise gezeichnet werden, weil entweder nur stroke
		// oder fill ausgefhrt wird. 
		canvas.arc(radius, radius, radius - 2, 0, 
				(float) (2* Math.PI), false);
		canvas.setFillStyle("#E8EEF7");
		canvas.fill();
		return canvas;
	}
	
	public int getAbsoluteEdgeConnectionPointY() {
		return WidgetUtilities.getAbsoluteMiddle(this);
	}
	
	protected void nodeComponentChanged() {
		redraw();
	}
	
	private void redraw() {
		_panel.remove(_circle);
		_circle = createCircle();;

		setPixelSize(calculateCircleDiameter(), calculateCircleDiameter());

		// Das Hinzufgen und Positionieren des Kreises muss nacheinander
		// geschehen, weil sonst die Methoden calculateCentralNodeViewCenterPosition
		// und calculateCentralNodeViewMiddlePosition nicht richtig funktionieren.
		// Diese Methoden bentigen Breite und Hhe des Kreises. Komponenten
		// besitzen aber eine Gre erst nachdem die zu einem Container hinzugefgt
		// wurden und auf dem Bildschirm sichtbar sind.
		_panel.add(_circle, 0, 0);
		_panel.setWidgetPosition(getNodeComponent(),
				calculateNodeComponentCenterPosition(),
				calculateNodeComponentMiddlePosition());
	}

}
