package mima.presentation.gwt.nodeframes;

import mima.presentation.gwt.nodecomponent.NodeComponent;
import mima.presentation.gwt.nodecomponent.NodeFrame;
import mima.presentation.gwt.util.WidgetUtilities;

import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.Widget;
import com.gwt.components.client.Canvas;


/**
 * @author Virginio Carfagno
 */
public class RectangleFrame extends NodeFrame {

	private static final int PADDING = 20; 
	
	private AbsolutePanel _panel;
	private Widget _rectangle;
	
	
	public RectangleFrame(NodeComponent nodeComponent) {
		super(nodeComponent);
		_panel = new AbsolutePanel();
		_panel.add(nodeComponent);
		initWidget(_panel);
	}
	

	private int calculateNodeComponentCenterPosition() {
		return (calculateRectangleWidth() - getNodeComponent().getOffsetWidth()) / 2;
	}
	
	private int calculateNodeComponentMiddlePosition() {
		return (calculateRectangleHeight() - getNodeComponent().getOffsetHeight()) / 2;
	}

	private int calculateRectangleHeight() {
		return getNodeComponent().getOffsetHeight() + PADDING;
	}

	private int calculateRectangleWidth() {
		return getNodeComponent().getOffsetWidth() + PADDING;
	}

	private Widget createRectangle() {
		int width = calculateRectangleWidth();
		int height = calculateRectangleHeight();
		Canvas canvas = new Canvas(width, height);
		canvas.setFillStyle("yellow");
		canvas.fillRect(0, 0, width-1, height-1);
		canvas.rect(0, 0, width-1, height-1);
		canvas.setStrokeColor("green");
		canvas.stroke();
		return canvas;
	}
	
	public int getAbsoluteEdgeConnectionPointY() {
		return WidgetUtilities.getAbsoluteMiddle(this);
	}
	
	protected void nodeComponentChanged() {
		redraw();
	}
	
	private void redraw() {
		_panel.remove(_rectangle);
		_rectangle = createRectangle();

		setPixelSize(calculateRectangleWidth(), calculateRectangleHeight());

		// Das Hinzufgen und Positionieren des Kreises muss nacheinander
		// geschehen, weil sonst die Methoden calculateCentralNodeViewCenterPosition
		// und calculateCentralNodeViewMiddlePosition nicht richtig funktionieren.
		// Diese Methoden bentigen Breite und Hhe des Kreises. Komponenten
		// besitzen aber eine Gre erst nachdem die zu einem Container hinzugefgt
		// wurden und auf dem Bildschirm sichtbar sind.
		_panel.add(_rectangle, 0, 0);
		_panel.setWidgetPosition(getNodeComponent(),
				calculateNodeComponentCenterPosition(),
				calculateNodeComponentMiddlePosition());
	}

}
