package mima.domain;

import junit.framework.TestCase;


/**
 * @author Virginio Carfagno
 */
public abstract class NodeTest extends TestCase {

	private boolean _isNodeChangedCalled;
	
	private Node _node;


	protected abstract Node createNode();
	
	protected void setUp() throws Exception {
		_node = createNode();
	}
	
	public void testCreateNode() {
		assertEquals(null, _node.getLink());
		assertEquals(null, _node.getNote());
		assertTrue(_node.isExpanded());
		assertFalse(_node.hasSubNodes());
	}
	
	public void testExpandAll() {
		_node.toggleExpansion();
		
		SubNode subNode1 = new SubNode(_node);
		subNode1.toggleExpansion();
		
		SubNode subNode2 = new SubNode(subNode1);
		subNode2.toggleExpansion();
		
		SubNode subNode3 = new SubNode(subNode1);
		subNode3.toggleExpansion();
		
		assertFalse(_node.isExpanded());
		assertFalse(subNode1.isExpanded());
		assertFalse(subNode2.isExpanded());
		
		_node.expandAll();
		assertTrue(_node.isExpanded());
		assertTrue(subNode1.isExpanded());
		assertTrue(subNode2.isExpanded());
		assertTrue(subNode3.isExpanded());	
	}
	
	public void testHasSubNodes() {
		SubNode subNode = new SubNode(_node);
		assertTrue(_node.hasSubNodes());
		
		subNode.removeFromSuperNode();
		assertFalse(_node.hasSubNodes());
	}
	
	public void testObserver() {
		NodeObserver nodeObserver = new NodeObserver() {
			public void nodeChanged() {
				_isNodeChangedCalled = true;
			}
		};
		
		_node.registerObserver(nodeObserver);
		
		_isNodeChangedCalled = false;
		_node.setTitle("title");
		assertEquals(true, _isNodeChangedCalled);
		
		_isNodeChangedCalled = false;
		_node.setLink("link");
		assertEquals(true, _isNodeChangedCalled);
		
		_isNodeChangedCalled = false;
		_node.setNote("note");
		assertEquals(true, _isNodeChangedCalled);
		
		_isNodeChangedCalled = false;
		_node.setFrame(Node.Frame.RECTANGLE);
		assertEquals(true, _isNodeChangedCalled);
		
		_isNodeChangedCalled = false;
		SubNode subNode = new SubNode(_node);
		assertEquals(true, _isNodeChangedCalled);
		
		_isNodeChangedCalled = false;
		subNode.removeFromSuperNode();
		assertEquals(true, _isNodeChangedCalled);
		
		_node.deregisterObserver(nodeObserver);
		
		_isNodeChangedCalled = false;
		_node.setTitle("title");
		assertEquals(false, _isNodeChangedCalled);
	}
	
	public void testSetFrame() {
		_node.setFrame(Node.Frame.CIRCLE);
		assertEquals(Node.Frame.CIRCLE, _node.getFrame());
	}
	
	public void testToggleExpand() {
		assertTrue(_node.isExpanded());
		
		_node.toggleExpansion();
		assertFalse(_node.isExpanded());
		
		_node.toggleExpansion();
		assertTrue(_node.isExpanded());
	}
	
}
