/*
 * Copyright 2006 Google Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.google.gwt.xml.client.impl;

import com.google.gwt.xml.client.DOMException;

/**
 * Thrown when parse errors occur in the underlying implementation.
 */
public class DOMParseException extends DOMException {

  static String summarize(String text) {
    return text.substring(0, Math.min(text.length(), 128));
  }

  public DOMParseException() {
    super(DOMNodeException.SYNTAX_ERR, "Parse error");
  }

  public DOMParseException(String contents) {
    super(DOMNodeException.SYNTAX_ERR, "Failed to parse: "
      + summarize(contents));
    this.contents = contents;
  }

  public DOMParseException(String contents, Throwable e) {
    super(DOMNodeException.SYNTAX_ERR, "Failed to parse: "
      + summarize(contents));
    initCause(e);
    this.contents = contents;
  }

  public String getContents() {
    return contents;
  }

  private String contents;

}
